//===============================================================================================================================================================================//

exec("./data/shared.cs");
exec("./data/client/clPlayerItemHud.cs");

//===============================================================================================================================================================================//

function PlayerItem_GetSlotObjectViewer(%slotIndex, %create)
{
	if (!isObject($HUD_ToolIcon[%slotIndex]))
		return -1;
	
	// If it already exists, then we're done here.
	if (isObject($PlayerItemObject[%slotIndex]))
		return $PlayerItemObject[%slotIndex];
	
	// Determine if we should create it
	if (%create !$= "" && (%create == false || %create $= "false" || %create $= "0"))
		return -1;
	
	// Allocate the new object into the slot index
	$PlayerItemObject[%slotIndex] = new GuiObjectView()
	{
		position    = "0 0";
		extent      = $HUD_ToolIcon[%slotIndex].getExtent();
		horizSizing = "width";
		vertSizing  = "height";
	};
	
	// Add it to the slot
	$HUD_ToolIcon[%slotIndex].add($PlayerItemObject[%slotIndex]);
	
	// Done!
	return $PlayerItemObject[%slotIndex];
}

function PlayerItem_RenderPlayer(%slotIndex)
{
	if (!isObject(%objView = PlayerItem_GetSlotObjectViewer(%slotIndex, false)))
		return;
	
	// Validate data
	if (%objView.nodeData $= "" || %objView.colorData $= "" || %objView.colorTableCount $= "")
	{
		//error("Not enough data to render player!");
		//error("nodeData        = \"" @ %objView.nodeData @ "\"");
		//error("colorData       = \"" @ %objView.colorData @ "\"");
		//error("colorTableCount = \"" @ %objView.colorTableCount @ "\"");
		return;
	}
	
	// $HUD_ToolIcon[index]		The bitmap UI element for this tool.
	// $ToolData[index]		Datablock IDs for all our tools. Index using the slot number
	// $CurrScrollToolSlot		The current slot
	// $PlayerItemObject[index]	The GuiObjectView belonging to a specific slot, if any.
	
	// Set this to be nothing.
	$HUD_ToolIcon[%slotIndex].setBitmap("Add-Ons/Item_Player/img/Icon_PlayerItem_NoPlayer");
	
	// Setup the object
	%objView.setObject("", "base/data/shapes/player/m.dts", "", "");
	
	// Complain at every chance we get
	if (strLen(%nodeData) < $HeldPlayerItem::PlayerNode)
		warn("WARNING: PlayerItem_RenderPlayer() - Node data length does not match our total player nodes! Might be malformed data!");
	
	// Hide all nodes
	for (%i = 0; %i < $HeldPlayerItem::PlayerNodeCount; %i++)
	{
		%nodeData = $HeldPlayerItem::PlayerNode[%i];
		%objView.hideNode("", getWord(%nodeData, 0));
	}
	
	// Ready the color data
	%colorData = "";
	%currIndex = 0;
	
	// Build the color data table
	for (%i = 0; %i < %objView.colorTableCount; %i++)
		%colorData = trim(%colorData TAB %objView.colorTable[%i]);
	
	// Parse node data
	for (%i = 0; %i < $HeldPlayerItem::PlayerNodeCount; %i++)
	{
		%nodeData    = $HeldPlayerItem::PlayerNode[%i];
		%nodeName    = getWord(%nodeData, 0);
		%nodeVisible = atoi(%i >= strLen(%objView.nodeData) ? "0" : getSubStr(%objView.nodeData, %i, 1));
		%nodeColor   = PlayerItem_IndiceKeyToColor(%colorData, getSubStr(%objView.colorData, %currIndex * 2, 2));
		
		// Don't need to hide it, since it's already hidden. Just skip this.
		if (!%nodeVisible)
			continue;
		
		// We hit a visible node. Increment the current index!
		%currIndex++;
		
		// Configure this node
		%objView.unhideNode("", %nodeName);
		%objView.setNodeColor("", %nodeName, %nodeColor);
	}
	
	// Now configure the camera view for this object
	%objView.setCameraRot(mDegToRad(20), mDegToRad(0), mDegToRad(140));
	%objView.forceFOV       = 30;
	%objView.ambientColor   = "0.6 0.6 0.57 1";
	%objView.lightDirection = "0 0.57735 0";
	
	// Start walking, buddy
	%objView.setSequence("", 0, "walk", 1);
	
	// Setup IFL frames
	%objView.setIFLFrame("", "face", %objView.faceName);
	%objView.setIFLFrame("", "decal", %objView.decalName);
}

//===============================================================================================================================================================================//

function clientCmdPlayerItem_ClearSlot(%slotIndex)
{
	// Stop if it already doesn't exist
	if (!isObject(%objView = PlayerItem_GetSlotObjectViewer(%slotIndex, false)))
		return;
	
	// Delete it!
	%objView.delete();
	$PlayerItemObject[%slotIndex] = 0;
}

function clientCmdPlayerItem_SetColorTable(%slotIndex, %nodeColorTable)
{
	if (!isObject(%objView = PlayerItem_GetSlotObjectViewer(%slotIndex)))
		return;
	
	// Configure our object view
	%objView.colorTableCount = getFieldCount(%nodeColorTable);
	
	// Loop through and set the colors
	for (%i = 0; %i < %objView.colorTableCount; %i++)
		%objView.colorTable[%i] = getField(%nodeColorTable, %i);
}

function clientCmdPlayerItem_SetSlotData(%slotIndex, %playerName, %nodeData, %nodeColorIndices, %faceName, %decalName)
{
	if (!isObject(%objView = PlayerItem_GetSlotObjectViewer(%slotIndex)))
		return;
	
	// Configure it
	%objView.playerName = %playerName;
	%objView.nodeData   = %nodeData;
	%objView.colorData  = %nodeColorIndices;
	%objView.faceName   = PlayerItem_FindFaceByName(%faceName);
	%objView.decalName  = PlayerItem_FindDecalByName(%decalName);
}

function clientCmdPlayerItem_Render(%slotIndex)
{
	// Render it!
	PlayerItem_RenderPlayer(%slotIndex);
}

function clientCmdPlayerItem_OpenMiniGame()
{
	if (!PlayerItemHeld_MinigameHud.isShown())
		PlayerItemHeld_MinigameHud.setShown(true);
}

function clientCmdPlayerItem_CloseMiniGame()
{
	if (PlayerItemHeld_MinigameHud.isShown() && !PlayerItemHeld_MinigameHud.hasLost)
		PlayerItemHeld_MinigameHud.setShown(false);
}

//===============================================================================================================================================================================//