if (!isObject(HeldPlayerItem))
{
	datablock ItemData(HeldPlayerItem)
	{
		// Basic Item Properties
		shapeFile  = "base/data/shapes/player/m.dts";
		mass       = 1;
		density    = 0.2;
		elasticity = 0.2;
		friction   = 0.6;
		emap       = true;
	
		//gui properties
		uiName       = "Soul Trap";
		iconName     = "Add-Ons/Item_Player/img/Icon_PlayerItem";
		doColorShift = false;
	
		 // Dynamic properties defined by the scripts
		image	= HeldPlayerImage;
		canDrop = true;
	};
}

if (!isObject(HeldPlayerImage))
{
	datablock ShapeBaseImageData(HeldPlayerImage)
	{
		// Basic Item properties
		shapeFile = "base/data/shapes/player/empty.dts";
		emap      = true;
	
		// Specify mount point & offset for 3rd person, and eye offset
		// for first person rendering.
		mountPoint = 0;
		offset     = "0 0 0";
		
		// When firing from a point offset from the eye, muzzle correction
		// will adjust the muzzle vector to point to the eye LOS point.
		// Since this weapon doesn't actually fire from the muzzle point,
		// we need to turn this off.  
		correctMuzzleVector = false;
	
		// Add the WeaponImage namespace as a parent, WeaponImage namespace
		// provides some hooks into the inventory system.
		className = "WeaponImage";
	
		// Projectile && Ammo.
		item           = HeldPlayerItem;
		ammo           = " ";
		projectile     = "";
		projectileType = "";
	
		//melee particles shoot from eye node for consistancy
		melee           = true;
		doRetraction    = false;
		//raise your arm up or not
		armReady        = true;
		showBricks      = false;
		doColorShift    = false;
		colorShiftColor = redKeyItem.colorShiftColor;
		
		// Images have a state system which controls how the animations
		// are run, which sounds are played, script callbacks, etc. This
		// state system is downloaded to the client so that clients can
		// predict state changes and animate accordingly.  The following
		// system supports basic ready->fire->reload transitions as
		// well as a no-ammo->dryfire idle state.
		
		// Initial start up state
		stateName[0]				= "Activate";
		stateTimeoutValue[0]			= 0.0;
		stateTransitionOnTimeout[0]		= "Ready";
	
		stateName[1]				= "Ready";
		stateTransitionOnTriggerDown[1]		= "PreFire";
		stateAllowImageChange[1]		= true;
	
		stateName[2]				= "PreFire";
		stateScript[2]				= "onPreFire";
		stateAllowImageChange[2]		= true;
		stateTimeoutValue[2]			= 0.01;
		stateTransitionOnTimeout[2]		= "Fire";
	
		stateName[3]				= "Fire";
		stateTransitionOnTimeout[3]		= "CheckFire";
		stateTimeoutValue[3]			= 0.15;
		stateFire[3]				= true;
		stateAllowImageChange[3]		= true;
		stateSequence[3]			= "Fire";
		stateScript[3]				= "onFire";
		stateWaitForTimeout[3]			= true;
		stateSequence[3]			= "Fire";
	
		stateName[4]				= "CheckFire";
		stateTransitionOnTriggerUp[4]		= "StopFire";
	
		stateName[5]				= "StopFire";
		stateTransitionOnTimeout[5]		= "Ready";
		stateTimeoutValue[5]			= 0.01;
		stateAllowImageChange[5]		= true;
		stateWaitForTimeout[5]			= true;
		stateSequence[5]			= "StopFire";
		stateScript[5]				= "onStopFire";
	};
}

function HeldPlayerItem::onAdd(%this, %obj)
{
	// Always call the parent function first
	ItemData::onAdd(%this, %obj);
	
	// After we do that, we need to offset the item and scale it
	%obj.setScale(VectorScale(%obj.getScale(), 0.25));
	
	// "Design" it
	PlayerItem_SetItemAvatar(%obj, PlayerItem_BuildSlotInfo("", -1));
}

function HeldPlayerItem::onRemove(%this, %obj)
{
	if (isObject(%client = %obj.heldClientId))
	{
		%player = %client.player;
		%client.applyBodyParts();
		%client.applyBodyColors();
		%player.unHideNode("headSkin");
		%player.setScale(%player.B4PlayerHeld_Scale);
		%player.setDataBlock(%player.B4PlayerHeld_DataBlock);
		if (isObject(%player.B4PlayerHeld_LastControl))
			%client.setControlObject(%player.B4PlayerHeld_LastControl);
		else
			%client.setControlObject(%player);
		%player.setTransform(%obj.getTransform());
	}
}

function HeldPlayerItem::onPickup(%this, %obj, %user, %amount)
{
	// Enforce the canPickup rule
	if (!%obj.canPickup)
		return;
	
	// Require a client to pickup
	if (!isObject(%user.client))
		return;
	
	// Rank restriction
	if (!PlayerItem_GetIsAllowedToUse(%user.client))
	{
		if (getSimTime() - %user.client.lastPlayerItemWarning >= 4000)
		{
			%user.client.lastPlayerItemWarning = getSimTime();
			messageClient(%user.client, '', PlayerItem_GetRestrictLevelText());
		}
		return;
	}
	
	// No bots!
	if (%user.client.getClassName() !$= "GameConnection")
		return;
	
	// Stop here if we already have it.
	for (%i = 0; %i < %user.dataBlock.maxTools; %i++)
		if (isObject(%user.tool[%i]) && %user.tool[%i].getId() == HeldPlayerItem.getId())
			return;
	
	// Extract the held client ID before we banish the item
	%heldClient       = %obj.heldClientId;
	%obj.heldClientId = 0;
	
	// Run base code
	ItemData::onPickup(%this, %obj, %user, %amount);
	
	// Get the equipped index
	for (%i = 0; %i < %user.dataBlock.maxTools; %i++)
		if (isObject(%user.tool[%i]) && %user.tool[%i].getId() == HeldPlayerItem.getId())
			break;
	
	// Stop if we actually didn't equip it
	if (%i == %user.dataBlock.maxTools)
		return;
	
	// This will always be the first time this client picks this up. We know this because we previously
	// checked if the HeldPlayerItem was already in the player's inventory. If it was, then we wouldn't
	// have gotten to this point.
	
	// Reset the held player for this object
	%user.toolHeldClient[%i] = %heldClient;
	
	// Set the held client's camera
	if (isObject(%heldClient) && isObject(%heldClient.camera))
	{
		%heldClient.player.beingHeldBy     = %obj.client;
		%heldClient.player.beingHeldBySlot = %i;
		%heldClient.player.heldByItem      = 0;
		%heldClient.camera.setFlyMode();
		%heldClient.camera.mode = "Observer";
		%heldClient.setControlObject(%heldClient.camera);
		%heldClient.camera.setOrbitMode(%user, %user.getTransform(), 0.2, 10, 10, false);
	}
	
	// Send the client a player info
	%user.client.sendPlayerItemInfo(%i);
	
	if (isObject(%obj.spawnBrick))
	{
		// "Design" it
		schedule(%obj.spawnBrick.itemRespawnTime + 32, 0, PlayerItem_SetItemAvatar, %obj, PlayerItem_BuildSlotInfo("", -1));
	}
}

function HeldPlayerImage::onPreFire(%this, %obj, %slot)
{
	// Get the equipped index
	for (%slot = 0; %slot < %obj.dataBlock.maxTools; %slot++)
		if (isObject(%obj.tool[%slot]) && %obj.tool[%slot].getId() == HeldPlayerItem.getId())
			break;
	
	// Stop if we actually didn't equip it
	if (%slot == %player.dataBlock.maxTools)
		return;
	
	if (!isObject(%heldClient = %obj.toolHeldClient[%slot]))
	{
		%obj.throwingPlayer = false;
		return;
	}
	
	%obj.lastPlayerThrowTime = 0;
	cancel(%obj.throwHeldPlayer);
	%obj.throwHeldPlayer = %this.schedule(300, onStartThrowPlayer, %obj, %slot);
}

function HeldPlayerImage::onStartThrowPlayer(%this, %obj, %slot)
{
	// Get the equipped index
	for (%slot = 0; %slot < %obj.dataBlock.maxTools; %slot++)
		if (isObject(%obj.tool[%slot]) && %obj.tool[%slot].getId() == HeldPlayerItem.getId())
			break;
	
	// Stop if we actually didn't equip it
	if (%slot == %player.dataBlock.maxTools)
		return;
	
	if (!isObject(%heldClient = %obj.toolHeldClient[%slot]))
	{
		%obj.throwingPlayer = false;
		return;
	}
	
	%obj.throwingPlayer      = true;
	%obj.lastPlayerThrowTime = getSimTime();
	%obj.playThread(1, "armReadyRight");
	%obj.playThread(2, "spearReady");
}

function HeldPlayerImage::onStopFire(%this, %obj, %slot)
{
	if (isEventPending(%obj.throwHeldPlayer))
		cancel(%obj.throwHeldPlayer);
	
	if (!%obj.throwingPlayer)
		return;
	
	// Get the equipped index
	for (%slot = 0; %slot < %obj.dataBlock.maxTools; %slot++)
		if (isObject(%obj.tool[%slot]) && %obj.tool[%slot].getId() == HeldPlayerItem.getId())
			break;
	
	// Stop if we actually didn't equip it
	if (%slot == %player.dataBlock.maxTools)
		return;
	
	if (!isObject(%heldClient = %obj.toolHeldClient[%slot]))
		return;
	
	// Don't throw them if the animation didn't play
	if (getSimTime() - %obj.lastPlayerThrowTime < 910)
	{
		%obj.playThread(2, "root");
		%obj.playThread(1, "armReadyRight");
		return;
	}
	
	// THROW THEM.
	%obj.playThread(2, "spearThrow");
	%obj.client.PlaceOtherPlayer(%slot, VectorAdd(%obj.getEyePoint(), VectorScale(VectorNormalize(%obj.getVelocity()), VectorLen(%obj.getScale()))));
	%heldClient.player.setVelocity(VectorAdd(%obj.getVelocity(), VectorScale(%obj.getEyeVector(), 45)));
	%obj.playThread(1, "root");
}

function HeldPlayerImage::onFire(%this, %player, %slot)
{
	// Get the equipped index
	for (%slot = 0; %slot < %player.dataBlock.maxTools; %slot++)
		if (isObject(%player.tool[%slot]) && %player.tool[%slot].getId() == HeldPlayerItem.getId())
			break;
	
	// Stop if we actually didn't equip it
	if (%slot == %player.dataBlock.maxTools)
		return;
	
	%start = %player.getEyePoint();
	%vec   = vectorScale(%player.getMuzzleVector(%slot), 10 * getWord(%player.getScale(), 2));
	%end   = vectorAdd(%start, %vec);
	%mask  = $TypeMasks::InteriorObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::FxBrickObjectType;
	
	if (!isObject(%player.toolHeldClient[%slot]))
		%mask = $TypeMasks::PlayerObjectType;
	
	%rayCast = containerRayCast(%start, %end, %mask, %player);
	
	if (!%rayCast)
		return;
	
	%hitObj    = getWord(%rayCast, 0);
	%hitPos    = getWords(%rayCast, 1, 3);
	%hitNormal = getWords(%rayCast, 4, 6);
	
	%this.onHitObject(%player, %slot, %hitObj, %hitPos, %hitNormal);
}

function HeldPlayerImage::onHitObject(%this, %player, %slot, %hitObj, %hitPos, %hitNormal)
{
	if (!isObject(%player.toolHeldClient[%slot]))
	{
		if (%hitObj.getClassName() !$= "Player")
			return;
		
		// Got a player! Pick them up!
		%player.client.PickUpOtherPlayer(%hitObj, %slot);
		
		HeldPlayerImage::onMount(%this, %player, %slot);
		return;
	}
	
	if (VectorDot(%hitNormal, "0 0 1") <= 0.5)
	{
		messageClient(%player.client, '', "\c2You gotta place them on solid ground.");
		return;
	}
	
	if (isEventPending(%player.throwHeldPlayer))
		cancel(%player.throwHeldPlayer);
	
	%player.playThread(1, root);
	%player.playThread(2, root);
	%player.throwingPlayer = false;
	
	// Place them back down.
	%player.client.PlaceOtherPlayer(%slot, %hitPos);
	
	//if (isObject(%hitObj))
	//	announce("itemPlayer.cs: Hit Object   = " @ %hitObj.getName() @ " (" @ %hitObj.getId() @ ")");
	//else
	//	announce("itemPlayer.cs: Hit Object   = <null>");
	//announce("itemPlayer.cs: Hit Normal   = " @ %hitNormal SPC "(" @ VectorCross(%hitNormal, "0 0 1") @ ")");
	//announce("itemPlayer.cs: Hit Position = " @ %hitPos);
}

function HeldPlayerImage::onMount(%this, %obj, %slot)
{
	if (isObject(%obj.beingHeldBy))
		return;
	
	// Get the equipped index
	for (%slot = 0; %slot < %obj.dataBlock.maxTools; %slot++)
		if (isObject(%obj.tool[%slot]) && %obj.tool[%slot].getId() == HeldPlayerItem.getId())
			break;
	
	// Stop if we actually didn't equip it
	if (%slot == %player.dataBlock.maxTools)
		return;
	
	if (!PlayerItem_GetIsAllowedToUse(%obj.client))
	{
		serverCmdDropTool(%obj.client, %slot);
		return;
	}
	
	if (!isObject(%heldClient = %obj.toolHeldClient[%slot]))
	{
		%obj.playThread(1, root);
		%obj.setFaceName("memeHappy");
		if ($Pref::Server::HoldablePlayer::PlaySound)
			%obj.playAudio(0, glassExplosionSound);
		
		return;
	}
	
	%heldClient.setControlObject(%heldClient.player);
	%heldClient.player.canDismount = false;
	
	%obj.playThread(2, armReadyRight);
	%heldClient.player.setScale("0.2 0.2 0.2");
	%heldClient.player.setTransform(%obj.getTransform());
	%obj.mountObject(%heldClient.player, HeldPlayerImage.mountPoint);
	%heldClient.applyBodyParts();
	%heldClient.applyBodyColors();
	%heldClient.player.unHideNode("headSkin");
	%heldClient.player.setShapeName(%heldClient.getPlayerName(), 8564862);
}

function HeldPlayerImage::onUnMount(%this, %obj, %slot)
{
	// Fix the arm
	schedule(10, 0, fixArmReady, %obj);
	
	// Get the equipped index
	for (%slot = 0; %slot < %obj.dataBlock.maxTools; %slot++)
		if (isObject(%obj.tool[%slot]) && %obj.tool[%slot].getId() == HeldPlayerItem.getId())
			break;
	
	// Stop if we actually didn't equip it
	if (%slot == %player.dataBlock.maxTools)
		return;
	
	if (!isObject(%heldClient = %obj.toolHeldClient[%slot]))
	{
		%obj.setFaceName(%obj.client.faceName);
		if ($Pref::Server::HoldablePlayer::PlaySound)
			%obj.stopAudio(0);
		return;
	}
	
	// Force them to spectate the player holding them
	if (isObject(%otherCam = %heldClient.camera))
	{
		%otherCam.setFlyMode();
		%otherCam.mode = "Observer";
		%otherCam.setOrbitMode(%obj, %obj.getTransform(), 0.2, 10, 10, false);
		%heldClient.setControlObject(%otherCam);
	}
	
	if (isEventPending(%obj.throwHeldPlayer))
		cancel(%obj.throwHeldPlayer);
	
	%obj.playThread(1, root);
	%obj.playThread(2, root);
	%obj.throwingPlayer = false;
	%heldClient.player.setScale("0.01 0.01 0.01");
	%heldClient.player.dismount();
	%heldClient.player.setTransform("0 0 0");
	%heldClient.player.setShapeName("", 8564862);
	%heldClient.player.hideNode("ALL");
}