// %client.player.heldByItem			<< The object ID of the dropped item we're confined to.
// %client.player.beingHeldBy			<< The object ID of the client we're being held by.
// %client.player.beingHeldBySlot		<< The slot index for the holder's HeldPlayerItem which we are bound to.
// %client.player.toolHeldClient[%slotIndex]	<< The object ID of the client we're holding in this tool slot.
// %client.player.B4PlayerHeld_Scale            << Our original scale before we were picked up.
// %client.player.B4PlayerHeld_DataBlock        << Our original datablock before we were picked up.
// %client.player.B4PlayerHeld_LastControl      << Our last control object before we were picked up.
// %client.player.isStrugglingPlayerItem	<< Whether we are struggling to get out of our holder's grasp.

package HoldablePlayerItemServerPackage
{
	// Purpose: Dropping the HeldPlayerItem item should detail the dropped item with the held client's avatar, name, and should also
	//          set the held client's camera.
	function ServerCmdDropTool(%client, %position)
	{
		if (!isObject(%player = %client.player) || !isObject(%item = %player.tool[%position]) || !%item.canDrop)
			return;
		
		// We don't care if it isn't our thing we're handling
		if (%item.getName() !$= "HeldPlayerItem")
		{
			Parent::ServerCmdDropTool(%client, %position);
			return;
		}
		
		%heldClient   = %player.toolHeldClient[%position];
		%zScale       = getWord(%player.getScale(), 2);
		%muzzlepoint  = VectorAdd(%player.getPosition(), "0 0" SPC 1.5 * %zScale);
		%muzzlevector = %player.getEyeVector();
		%muzzlepoint  = VectorAdd(%muzzlepoint, %muzzlevector);
		%playerRot    = rotFromTransform(%player.getTransform());
		
		// Allocate the new item
		%thrownItem = new Item()
		{
			dataBlock = %item;
		};
		
		// Setup scale
		%thrownItem.setScale(VectorScale(%player.getScale(), 0.45));
		
		// Add it to the mission cleanup
		MissionCleanup.add(%thrownItem);
		
		// Setup everything
		%thrownItem.setTransform(%muzzlepoint @ " " @ %playerRot);
		%thrownItem.setVelocity(VectorScale(%muzzlevector, 20 * %zScale));
		//if (!isObject(%heldClient))
			%thrownItem.schedulePop();
		
		// Configure it
		%thrownItem.miniGame     = %client.miniGame;
		%thrownItem.bl_id        = %client.getBLID();
		%thrownItem.heldClientId = %heldClient;
		%thrownItem.setCollisionTimeout(%player);
		
		// If it's a weapon, 
		if (%item.className $= "Weapon")
			%player.weaponCount -= 1;

		// Decorate the player
		PlayerItem_SetItemAvatar(%thrownItem, PlayerItem_BuildSlotInfo(%player.client, %position));
		
		// Unmount the image
		if (%player.getMountedImage(%item.image.mountPoint) > 0)
		{
			if (%player.getMountedImage(%item.image.mountPoint).getId() == %item.image.getId())
			{
				%player.unmountImage(%item.image.mountPoint);
				HeldPlayerImage::onUnMount(%this);
			}
		}
		
		// Update the held client's camera!
		if (isObject(%heldClient))
		{
			if (isObject(%heldClient.camera))
			{
				%heldClient.camera.setFlyMode();
				%heldClient.camera.mode = "Observer";
				%heldClient.setControlObject(%heldClient.camera);
				%heldClient.camera.setOrbitMode(%thrownItem, %thrownItem.getTransform(), 0.2, 10, 10, false);
			}
			
			// Update their player
			%heldClient.player.beingHeldBy     = 0;
			%heldClient.player.beingHeldBySlot = -1;
			%heldClient.player.heldByItem      = %thrownItem.getId();
			
			// Tell the held client to stop struggling
			%heldClient.setStrugglingPlayerItem(false);
		}
		
		// Set its stuff in the given slot to nothing
		%player.tool[%position]           = 0;
		%player.toolHeldClient[%position] = 0;
		
		// Message them that they've given up the slot
		messageClient(%client, 'MsgItemPickup', '', %position, 0);
		
		// And finally, tell them to clear the slot
		commandToClient(%client, 'PlayerItem_ClearSlot', %position);
	}
	
	// Purpose: Dying whilst holding a player should make the held player no longer held.
	//          Dying whilst being held should make the holder stop holding you.
	function Armor::onDisabled(%this, %obj, %state)
	{
		// Forcibly close the minigame.
		%obj.client.setStrugglingPlayerItem(false);
		
		// Apply the disable
		Parent::onDisabled(%this, %obj, %state);
		
		// Stop if we're not dead.
		if (%obj.getState() !$= "Dead" || !isObject(%client = %obj.client))
			return;
		
		// If we're attached to an item, delete it.
		if (isObject(%item = %obj.heldByItem))
		{
			%obj.client.applyBodyParts();
			%obj.client.applyBodyColors();
			%obj.unHideNode("headSkin");
			%obj.setScale("1 1 1");
			%obj.setTransform(%item.getTransform());
			%item.heldClientId = 0;
			%item.delete();
		}
		
		// Force our holder to drop us
		if (isObject(%holder = %obj.beingHeldBy))
		{
			if (%holder.player.isImageMounted(HeldPlayerImage))
			{
				%holder.player.setFaceName("memeHappy");
				%holder.player.playThread(1, activate2);
				%holder.player.playThread(2, root);
			}
			
			// Cancel the throw event
			if (isEventPending(%holder.player.throwHeldPlayer))
				cancel(%holder.player.throwHeldPlayer);
			
			// Stop throwing the player.
			%holder.player.throwingPlayer = false;
			%holder.PlaceOtherPlayer(%obj.beingHeldBySlot, getWords(%holder.player.getTransform(), 0, 3));
		}
		
		// Set this for now
		%oldValue      = %client.player;
		%client.player = %obj;
		
		// Remove all of our held players
		for (%i = 0; %i < %obj.dataBlock.maxTools; %i++)
		{
			// Skip holdless slots
			if (!isObject(%heldClient = %obj.toolHeldClient[%i]))
				continue;
			
			// Drop them!!! Please!!!
			%client.player.mountImage(HeldPlayerImage, %i);
			%client.PlaceOtherPlayer(%i, getWords(%obj.getTransform(), 0, 3));
		}
		
		// Now reset it back
		%client.player = %oldValue;
	}
	
	// Purpose: Leaving the game whilst holding a player should reset the held player's "Being Held" state.
	//          Leaving the game whilst being held should reset the holder's "holding player" state.
	function GameConnection::onClientLeaveGame(%this)
	{
		Parent::onClientLeaveGame(%this);
		
		// Forcibly close the minigame.
		%this.setStrugglingPlayerItem(false);
		
		// MUST have a player object.
		if (!isObject(%player = %this.player))
			return;
		
		// If we're attached to an item, delete it.
		if (isObject(%item = %player.heldByItem))
		{
			%item.heldClientId = 0;
			%item.delete();
		}
		
		// Force our holder to drop us
		if (isObject(%holder = %player.beingHeldBy))
		{
			if (%holder.player.isImageMounted(HeldPlayerImage))
			{
				%holder.setFaceName("memeHappy");
				%holder.playThread(1, activate2);
				%holder.playThread(2, root);
			}
			
			// Cancel the throw event
			if (isEventPending(%holder.player.throwHeldPlayer))
				cancel(%holder.player.throwHeldPlayer);
			
			// Stop throwing the player.
			%holder.player.throwingPlayer = false;
			
			// Forcefully plant them down
			%holder.PlaceOtherPlayer(%player.beingHeldBySlot, getWords(%holder.player.getTransform(), 0, 3));
		}
		
		// Remove all of our held players
		for (%i = 0; %i < %player.dataBlock.maxTools; %i++)
		{
			// Skip holdless slots
			if (!isObject(%heldClient = %player.toolHeldClient[%i]))
				continue;
			
			// Drop them!!! Please!!!
			%player.mountImage(HeldPlayerImage, %i);
			%this.PlaceOtherPlayer(%i, getWords(%player.getTransform(), 0, 3));
		}
	}
	
	// Purpose: Clicking while being held should start a struggle minigame.
	function Player::ActivateStuff(%player)
	{
		// Always run this first
		Parent::ActivateStuff(%player);
		
		// Struggle minigame
		if (isObject(%player.beingHeldBy) && isObject(%player.client))
			%player.client.setStrugglingPlayerItem(true);
	}
};

activatePackage(HoldablePlayerItemServerPackage);