// Node format:
// <nodeName> <nodeColorVarName> <isVisibleMethod> <varName> <globalVarName>

//===============================================================================================================================================================================//

function PlayerItem_FindFaceByName(%faceName)
{
	for (%i = 0; %i < 65535; %i++)
	{
		%faceFile = $face[%i];
		
		// Stop if we've reached the end
		if (%faceFile $= "")
			break;
		
		// Compare the face names
		if (fileBase(%faceFile) !$= %faceName)
			continue;
		
		// Found it!
		return %i;
	}
	
	return 0;
}

function PlayerItem_FindDecalByName(%decalName)
{
	for (%i = 0; %i < 65535; %i++)
	{
		%decalFile = $decal[%i];
		
		// Stop if we've reached the end
		if (%decalFile $= "")
			break;
		
		// Compare the face names
		if (fileBase(%decalFile) !$= %decalName)
			continue;
		
		// Found it!
		return %i;
	}
	
	return 0;
}

function PlayerItem_NodeHelper_GetIsVisible(%client, %nodeIndex)
{
	%nodeData         = $HeldPlayerItem::PlayerNode[%nodeIndex];
	%nodeName         = getWord(%nodeData, 0);
	%nodeColor        = getWord(%nodeData, 1);
	%method           = getWord(%nodeData, 2);
	%methodArgs       = removeWord(removeWord(removeWord(%nodeData, 0), 0), 0);
	
	// Determine which method to use
	switch$(%method)
	{
		//-------------------------------------------------
		case "ALWAYS_HIDDEN":
			return false;
		
		//-------------------------------------------------
		case "ALWAYS_VISIBLE":
			return true;
		
		//-------------------------------------------------
		case "COMPARE":
			return eval("return (" @ %methodArgs @ ");");
		
		//-------------------------------------------------
		default:
			error("ERROR: PlayerItem_NodeHelper_GetIsVisible() - Unhandled GetIsVisible method \"" @ %method @ "\" for node \"" @ %nodeName @ "\".");
	}
	
	// Invalid method
	return false;
}

function PlayerItem_FindNodeIndexByName(%desiredNodeName)
{
	for (%i = 0; %i < $HeldPlayerItem::PlayerNodeCount; %i++)
	{
		%nodeData   = $HeldPlayerItem::PlayerNode[%i];
		%nodeName   = getWord(%nodeData, 0);
		%nodeColor  = getWord(%nodeData, 1);
		%method     = getWord(%nodeData, 2);
		%methodArgs = removeWord(removeWord(removeWord(%nodeData, 0), 0), 0);
		
		// Skip mismatches
		if (%nodeName !$= %desiredNodeName)
			continue;
		
		// Found it!
		return %i;
	}
	
	// Didn't find it...
	return -1;
}

function PlayerItem_GetNodeIndexInList(%nodeData, %index)
{
	for (%i = 0; %i < getWordCount(%nodeData); %i++)
	{
		if (%index $= getWord(%nodeData, %i))
			return %i;
	}
	
	return -1;
}

function PlayerItem_AddColorToTable(%colorTable, %color)
{
	for (%i = 0; %i < getFieldCount(%colorTable); %i++)
	{
		if (%color $= getField(%colorTable, %i))
			return %colorTable;
	}
	
	return trim(%colorTable TAB %color);
}

function PlayerItem_ColorToIndiceKey(%colorTable, %color)
{
	%index = 0;
	
	// Find the index
	for (%i = 0; %i < getFieldCount(%colorTable); %i++)
	{
		if (%color $= getField(%colorTable, %i))
		{
			%index = %i;
			break;
		}
	}
	
	// Build the indice
	return getSubStr($HeldPlayerItem::IndiceKeyList, %index / 16, 1) @ getSubStr($HeldPlayerItem::IndiceKeyList, %index % 16, 1);
}

function PlayerItem_IndiceKeyToColor(%colorTable, %indice)
{
	// Build the indice
	%index  = strPos($HeldPlayerItem::IndiceKeyList, getSubStr(%indice, 0, 1)) * 16;
	%index += strPos($HeldPlayerItem::IndiceKeyList, getSubStr(%indice, 1, 1));
	
	return getField(%colorTable, mClamp(%index, 0, getFieldCount(%colorTable) - 1));
}

//===============================================================================================================================================================================//

$HeldPlayerItem::IndiceKeyList                                  = "0123456789ABCDEF";
$HeldPlayerItem::PlayerNodeCount                                = -1;
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "headSkin headColor ALWAYS_VISIBLE";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "helmet hatColor COMPARE $hat[%client.hat] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "copHat hatColor COMPARE $hat[%client.hat] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "knitHat hatColor COMPARE $hat[%client.hat] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "scoutHat hatColor COMPARE $hat[%client.hat] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "bicorn hatColor COMPARE $hat[%client.hat] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "pointyHelmet hatColor COMPARE $hat[%client.hat] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "flareHelmet hatColor COMPARE $hat[%client.hat] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "plume accentColor COMPARE $accent[%client.accent] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "triplume accentColor COMPARE $accent[%client.accent] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "septplume accentColor COMPARE $accent[%client.accent] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "visor accentColor COMPARE $accent[%client.accent] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "chest chestColor COMPARE $chest[%client.chest] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "femchest chestColor COMPARE $chest[%client.chest] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "armor packColor COMPARE $pack[%client.pack] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "pack packColor COMPARE $pack[%client.pack] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "quiver packColor COMPARE $pack[%client.pack] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "tank packColor COMPARE $pack[%client.pack] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "bucket packColor COMPARE $pack[%client.pack] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "cape packColor COMPARE $pack[%client.pack] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "epaulets secondPackColor COMPARE $secondPack[%client.secondPack] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "epauletsRankA secondPackColor COMPARE $secondPack[%client.secondPack] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "epauletsRankB secondPackColor COMPARE $secondPack[%client.secondPack] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "epauletsRankC secondPackColor COMPARE $secondPack[%client.secondPack] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "epauletsRankD secondPackColor COMPARE $secondPack[%client.secondPack] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "ShoulderPads secondPackColor COMPARE $secondPack[%client.secondPack] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "RArm rarmColor COMPARE $rarm[%client.rarm] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "RArmSlim rarmColor COMPARE $rarm[%client.rarm] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "LArm larmColor COMPARE $larm[%client.larm] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "LArmSlim larmColor COMPARE $larm[%client.larm] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "RHand rhandColor COMPARE $rhand[%client.rhand] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "RHook rhandColor COMPARE $rhand[%client.rhand] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "LHand lhandColor COMPARE $lhand[%client.lhand] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "LHook lhandColor COMPARE $lhand[%client.lhand] $= %nodeName";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "pants hipColor COMPARE $Hip[%client.hip] !$= \"skirtHip\"";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "RShoe rlegColor COMPARE $RLeg[%client.rleg] $= %nodeName && $Hip[%client.hip] !$= \"skirtHip\"";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "RPeg rlegColor COMPARE $RLeg[%client.rleg] $= %nodeName && $Hip[%client.hip] !$= \"skirtHip\"";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "LShoe llegColor COMPARE $LLeg[%client.lleg] $= %nodeName && $Hip[%client.hip] !$= \"skirtHip\"";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "LPeg llegColor COMPARE $LLeg[%client.lleg] $= %nodeName && $Hip[%client.hip] !$= \"skirtHip\"";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "RSki SPECIAL_CURRENT_COLOR COMPARE %client.wasOnSkis";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "LSki SPECIAL_CURRENT_COLOR COMPARE %client.wasOnSkis";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "skirtHip hipColor COMPARE $Hip[%client.hip] $= \"skirtHip\"";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "SkirtTrimRight rlegColor COMPARE $Hip[%client.hip] $= \"skirtHip\"";
$HeldPlayerItem::PlayerNode[$HeldPlayerItem::PlayerNodeCount++] = "SkirtTrimLeft llegColor COMPARE $Hip[%client.hip] $= \"skirtHip\"";
$HeldPlayerItem::PlayerNodeCount++;
$HeldPlayerItem::DefaultPlayerNodeCount                                       = -1;
$HeldPlayerItem::DefaultPlayerNode[$HeldPlayerItem::DefaultPlayerNodeCount++] = "1 0.878 0.611 1 headSkin lhand rhand";
$HeldPlayerItem::DefaultPlayerNode[$HeldPlayerItem::DefaultPlayerNodeCount++] = "0.9 0 0 1 rarm larm";
$HeldPlayerItem::DefaultPlayerNode[$HeldPlayerItem::DefaultPlayerNodeCount++] = "1 1 1 1 chest";
$HeldPlayerItem::DefaultPlayerNode[$HeldPlayerItem::DefaultPlayerNodeCount++] = "0 0 1 1 pants lshoe rshoe";
$HeldPlayerItem::DefaultPlayerNodeCount++;